## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
from CvPythonExtensions import *
import CvUtil
import ScreenInput
import CvScreenEnums

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

class CvPediaTrait:
	"Civilopedia Screen for Traits"

	def __init__(self, main):
		self.iTrait = -1
		self.top = main

		self.X_MAIN_PANE = self.top.X_PEDIA_PAGE
		self.Y_MAIN_PANE = self.top.Y_PEDIA_PAGE
		self.W_MAIN_PANE = 210
		self.H_MAIN_PANE = 210

		self.W_ICON = 150
		self.H_ICON = 150
		self.X_ICON = self.X_MAIN_PANE + (self.H_MAIN_PANE - self.H_ICON) / 2
		self.Y_ICON = self.Y_MAIN_PANE + (self.H_MAIN_PANE - self.H_ICON) / 2
		self.ICON_SIZE = 64

		self.X_LEADERS = self.X_MAIN_PANE + self.W_MAIN_PANE + 18
		self.W_LEADERS = self.top.R_PEDIA_PAGE - self.X_LEADERS
		self.H_LEADERS = 110
		self.Y_LEADERS = self.Y_MAIN_PANE + self.H_MAIN_PANE - self.H_LEADERS

		self.X_SPECIAL = self.X_MAIN_PANE
		self.Y_SPECIAL = self.Y_MAIN_PANE + self.H_MAIN_PANE + 5
		self.W_SPECIAL = self.top.W_PEDIA_PAGE
		self.H_SPECIAL = (self.top.B_PEDIA_PAGE - self.Y_SPECIAL) * 2 / 3 - (5 // 2)

		self.X_TEXT = self.X_MAIN_PANE
		self.Y_TEXT = self.Y_SPECIAL + self.H_SPECIAL + 5
		self.W_TEXT = self.top.W_PEDIA_PAGE
		self.H_TEXT = self.top.B_PEDIA_PAGE - self.Y_TEXT

	# Screen construction function
	def interfaceScreen(self, iTrait):
		self.iTrait = iTrait

		self.top.deleteAllWidgets()

		screen = self.top.getScreen()

		bNotActive = (not screen.isActive())
		if bNotActive:
			self.top.setPediaCommonWidgets()

		# Header...
		szHeader = u"<font=4b>" + gc.getTraitInfo(self.iTrait).getDescription().upper() + u"</font>"
		szHeaderId = self.top.getNextWidgetName()
		screen.setLabel(szHeaderId, "Background", szHeader, CvUtil.FONT_CENTER_JUSTIFY, self.top.X_SCREEN, self.top.Y_TITLE, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		# Top
		screen.setText(self.top.getNextWidgetName(), "Background", self.top.MENU_TEXT, CvUtil.FONT_LEFT_JUSTIFY, self.top.X_MENU, self.top.Y_MENU, 0, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_PEDIA_MAIN, CivilopediaPageTypes.CIVILOPEDIA_PAGE_TRAIT, -1)

		if self.top.iLastScreen	!= CvScreenEnums.PEDIA_TRAIT or bNotActive:
			self.placeLinks(true)
			self.top.iLastScreen = CvScreenEnums.PEDIA_TRAIT
		else:
			self.placeLinks(false)

		# Icon
		screen.addPanel( self.top.getNextWidgetName(), "", "", False, False,
			self.X_MAIN_PANE, self.Y_MAIN_PANE, self.W_MAIN_PANE, self.H_MAIN_PANE, PanelStyles.PANEL_STYLE_BLUE50)
		screen.addPanel(self.top.getNextWidgetName(), "", "", false, false,
			self.X_ICON, self.Y_ICON, self.W_ICON, self.H_ICON, PanelStyles.PANEL_STYLE_MAIN)
		screen.addDDSGFC(self.top.getNextWidgetName(), gc.getTraitInfo(self.iTrait).getButton(),
			self.X_ICON + self.W_ICON/2 - self.ICON_SIZE/2, self.Y_ICON + self.H_ICON/2 - self.ICON_SIZE/2, self.ICON_SIZE, self.ICON_SIZE, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		self.placeLeaders()
		self.placeSpecial()
		self.placeText()

		return

	def placeLeaders(self):
		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()

		#Leaders who has the trait.
		szPanelTitle = localText.getText("TXT_KEY_PEDIA_CATEGORY_LEADER", ()) + localText.getText("TXT_KEY_OR", ()) + localText.getText("TXT_KEY_PEDIA_CATEGORY_CIV", ())
		screen.addPanel( panelName, szPanelTitle, "", False, True, self.X_LEADERS, self.Y_LEADERS, self.W_LEADERS, self.H_LEADERS, PanelStyles.PANEL_STYLE_BLUE50 )
		screen.attachLabel(panelName, "", "  ")
		for iLeader in range(gc.getNumLeaderHeadInfos()):
			pLeader = gc.getLeaderHeadInfo(iLeader)
			if pLeader.hasTrait(self.iTrait):
				self.iLeader = iLeader
				for iCivilization in range(gc.getNumCivilizationInfos()):
					if gc.getCivilizationInfo(iCivilization).isLeaders(iLeader):
						if not gc.getLeaderHeadInfo(iLeader).isGraphicalOnly():
							screen.attachImageButton(panelName, "", pLeader.getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_LEADER, iLeader, self.top.pickLeaderCiv(iLeader), False)
							break
		
		#Civilizations which has the trait.
		for iCivilization in range(gc.getNumCivilizationInfos()):
			pCivilization = gc.getCivilizationInfo(iCivilization)
			if pCivilization.getCivTrait() == self.iTrait:
				screen.attachImageButton(panelName, "", pCivilization.getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIV, iCivilization, 1, False)

	def placeSpecial(self):
		screen = self.top.getScreen()
		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, localText.getText("TXT_KEY_PEDIA_EFFECTS", ()), "", True, False, self.X_SPECIAL, self.Y_SPECIAL, self.W_SPECIAL, self.H_SPECIAL, PanelStyles.PANEL_STYLE_BLUE50 )
		szListName = self.top.getNextWidgetName()
		screen.attachListBoxGFC( panelName, szListName, "", TableStyles.TABLE_STYLE_EMPTY )
		screen.enableSelect(szListName, False)
		
		szSpecial = CyGameTextMgr().parseTraits(self.iTrait, CivilizationTypes.NO_CIVILIZATION, false, true)	#bool bTemporary, bool bCivilopediaText
		szSpecial = szSpecial.strip("\n")	#trim empty lines

		screen.addMultilineText(szListName, szSpecial, self.X_SPECIAL+5, self.Y_SPECIAL+30, self.W_SPECIAL-5, self.H_SPECIAL-32, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)

	def placeText(self):
		screen = self.top.getScreen()

		panelName = self.top.getNextWidgetName()
		screen.addPanel( panelName, "", "", true, true,
				 self.X_TEXT, self.Y_TEXT, self.W_TEXT, self.H_TEXT, PanelStyles.PANEL_STYLE_BLUE50 )
 
		szText = gc.getTraitInfo(self.iTrait).getCivilopedia()
		screen.attachMultilineText( panelName, "Text", szText, WidgetTypes.WIDGET_GENERAL, -1, -1, CvUtil.FONT_LEFT_JUSTIFY)

	def placeLinks(self, bRedraw):
		screen = self.top.getScreen()
		if bRedraw:
			screen.clearListBoxGFC(self.top.LIST_ID)

		listSorted = self.top.getSortedList( gc.getNumTraitInfos(), gc.getTraitInfo )

		iSelected = 0
		for iIndex, (szIndexName, iTrait, szDescription) in enumerate(listSorted):
			if not gc.getTraitInfo(iTrait).isGraphicalOnly():
				if bRedraw:
					screen.appendListBoxString(self.top.LIST_ID, szDescription, WidgetTypes.WIDGET_PEDIA_JUMP_TO_TRAIT, iTrait, 0, CvUtil.FONT_LEFT_JUSTIFY)
				if iTrait == self.iTrait:
					iSelected = iIndex

		screen.setSelectedListBoxStringGFC(self.top.LIST_ID, iSelected)


	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		return 0

