## Sid Meier's Civilization 4
## Described by Denev

from CvPythonExtensions import *

# globals
gc = CyGlobalContext()

def placeUnitsContents(screen, panelName, iTech, bPlayerContext = false):

	if bPlayerContext:
		iActiveCiv = gc.getGame().getActiveCivilizationType()
	else:
		iActiveCiv = CivilizationTypes.NO_CIVILIZATION

	# Units
	for iLoopUnit in range(gc.getNumUnitInfos()):
		if (isTechRequiredForUnit(iTech, iLoopUnit)):
			if gc.getUnitInfo(iLoopUnit).getProductionCost() < 0:
				continue
#			if gc.getUnitInfo(iLoopUnit).isGraphicalOnly():
#				continue
			if iActiveCiv != CivilizationTypes.NO_CIVILIZATION:
				iUnitClass = gc.getUnitInfo(iLoopUnit).getUnitClassType()
				if iLoopUnit != gc.getCivilizationInfo(iActiveCiv).getCivilizationUnits(iUnitClass):
					continue
				if gc.getUnitInfo(iLoopUnit).getPrereqCiv() not in (iActiveCiv, CivilizationTypes.NO_CIVILIZATION):
					continue
			szButton = gc.getUnitInfo(iLoopUnit).getUnitButtonWithCivArtStyle(iActiveCiv)
			screen.attachImageButton( panelName, "", szButton, GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_UNIT, iLoopUnit, 1, False )

def placeBuildingsContents(screen, panelName, iTech, bPlayerContext = false):

	if bPlayerContext:
		iActiveCiv = gc.getGame().getActiveCivilizationType()
	else:
		iActiveCiv = CivilizationTypes.NO_CIVILIZATION

	# Buildings
	for iLoopBuilding in range(gc.getNumBuildingInfos()):
		if (isTechRequiredForBuilding(iTech, iLoopBuilding)):
#			if gc.getBuildingInfo(iLoopBuilding).isGraphicalOnly():
#				continue
			if iActiveCiv != CivilizationTypes.NO_CIVILIZATION:
				iBuildingClass = gc.getBuildingInfo(iLoopBuilding).getBuildingClassType()
				if iLoopBuilding != gc.getCivilizationInfo(iActiveCiv).getCivilizationBuildings(iBuildingClass):
					continue
				if gc.getBuildingInfo(iLoopBuilding).getPrereqCiv() not in (iActiveCiv, CivilizationTypes.NO_CIVILIZATION):
					continue
			screen.attachImageButton( panelName, "", gc.getBuildingInfo(iLoopBuilding).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BUILDING, iLoopBuilding, 1, False )

	# Processes possible
	for iLoopProcess in range(gc.getNumProcessInfos()):
		if (gc.getProcessInfo(iLoopProcess).getTechPrereq() == iTech):
			screen.attachImageButton( panelName, "", gc.getProcessInfo(iLoopProcess).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_PROCESS_INFO, iTech, iLoopProcess, False )

def placeOthersContents(screen, panelName, iTech, bPlayerContext = false):

	if bPlayerContext:
		iActiveCiv = gc.getGame().getActiveCivilizationType()
	else:
		iActiveCiv = CivilizationTypes.NO_CIVILIZATION

	# Projects
	for iLoopProject in range(gc.getNumProjectInfos()):
		if (isTechRequiredForProject(iTech, iLoopProject)):
#			if gc.getProjectInfo(iLoopProject).isGraphicalOnly():
#				continue
			if iActiveCiv != CivilizationTypes.NO_CIVILIZATION:
				if gc.getProjectInfo(iLoopProject).getPrereqCivilization() not in (iActiveCiv, CivilizationTypes.NO_CIVILIZATION):
					continue
			screen.attachImageButton( panelName, "", gc.getProjectInfo(iLoopProject).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROJECT, iLoopProject, 1, False )

	# Free unit
	if gc.getTechInfo(iTech).getFirstFreeUnitClass() != UnitClassTypes.NO_UNITCLASS:
		if iActiveCiv == CivilizationTypes.NO_CIVILIZATION:
			iLoopUnit = gc.getUnitClassInfo(gc.getTechInfo(iTech).getFirstFreeUnitClass()).getDefaultUnitIndex()
		else:
			iLoopUnit = gc.getCivilizationInfo(iActiveCiv).getCivilizationUnits(gc.getTechInfo(iTech).getFirstFreeUnitClass())
		if iLoopUnit != UnitTypes.NO_UNIT:
			szButton = gc.getUnitInfo(iLoopUnit).getUnitButtonWithCivArtStyle(iActiveCiv)
			screen.attachImageButton( panelName, "", szButton, GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_FREE_UNIT, iLoopUnit, iTech, False )

	# Religions unlocked
	for iReligion in range(gc.getNumReligionInfos()):
		if gc.getReligionInfo(iReligion).getTechPrereq() == iTech:
			screen.attachImageButton( panelName, "", gc.getReligionInfo(iReligion).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_FOUND_RELIGION, iTech, iReligion, False )

	#Civics
	for iCivic in range(gc.getNumCivicInfos()):
		if iTech == gc.getCivicInfo(iCivic).getTechPrereq():
#			if gc.getCivicInfo(iCivic).isGraphicalOnly():
#				continue
			if iActiveCiv != CivilizationTypes.NO_CIVILIZATION:
				if gc.getCivicInfo(iCivic).getPrereqCivilization() not in (iActiveCiv, CivilizationTypes.NO_CIVILIZATION):
					continue
			screen.attachImageButton( panelName, "", gc.getCivicInfo(iCivic).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_CIVIC, iCivic, 1, False )

	#Spells
	for iSpell in range(gc.getNumSpellInfos()):
		if iTech == gc.getSpellInfo(iSpell).getTechPrereq():
			if gc.getSpellInfo(iSpell).isGraphicalOnly():
				continue
			if iActiveCiv != CivilizationTypes.NO_CIVILIZATION:
				if gc.getSpellInfo(iSpell).getCivilizationPrereq() not in (iActiveCiv, CivilizationTypes.NO_CIVILIZATION):
					continue
			screen.attachImageButton( panelName, "", gc.getSpellInfo(iSpell).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_SPELL, iSpell, 1, False )

	#Promotions
	for iPromotion in range(gc.getNumPromotionInfos()):
		if iTech == gc.getPromotionInfo(iPromotion).getTechPrereq():
#			if gc.getPromotionInfo(iPromotion).isGraphicalOnly():
#				continue
			screen.attachImageButton( panelName, "", gc.getPromotionInfo(iPromotion).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_PROMOTION, iPromotion, 1, False )

	#Improvements
	liUnlockabelImprovements = []
	for iBuild in range(gc.getNumBuildInfos()):
#		if gc.getBuildInfo(iBuild).isGraphicalOnly():
#			continue
		bTechFound = false
		if gc.getBuildInfo(iBuild).getTechPrereq() == TechTypes.NO_TECH:
			for iFeature in range(gc.getNumFeatureInfos()):
				if gc.getBuildInfo(iBuild).getFeatureTech(iFeature) == iTech:
					bTechFound = true
					break
		elif gc.getBuildInfo(iBuild).getTechPrereq() == iTech:
				bTechFound = true

		if (bTechFound):
			if gc.getBuildInfo(iBuild).getImprovement() == ImprovementTypes.NO_IMPROVEMENT:
				screen.attachImageButton( panelName, "", gc.getBuildInfo(iBuild).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_IMPROVEMENT, iTech, iBuild, False )
			else:
				screen.attachImageButton( panelName, "", gc.getBuildInfo(iBuild).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_IMPROVEMENT, gc.getBuildInfo(iBuild).getImprovement(), 1, False )
				liUnlockabelImprovements.append(gc.getBuildInfo(iBuild).getImprovement())

	#Resources Reveal
	liRevealedBonuses = []
	for iBonus in range(gc.getNumBonusInfos()):
		if iTech == gc.getBonusInfo(iBonus).getTechReveal():
#			if gc.getBonusInfo(iBonus).isGraphicalOnly():
#				continue
			screen.attachImageButton( panelName, "", gc.getBonusInfo(iBonus).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_HELP_BONUS_REVEAL, iTech, iBonus, False )
			liRevealedBonuses.append(iBonus)

	#Resources Enable
	for iBonus in range(gc.getNumBonusInfos()):
		if iTech == gc.getBonusInfo(iBonus).getTechCityTrade():
#			if gc.getBonusInfo(iBonus).isGraphicalOnly():
#				continue
			bCanImproveAndEnableTogether = false
			for iImprovement in liUnlockabelImprovements:
				if gc.getImprovementInfo(iImprovement).isImprovementBonusMakesValid(iBonus):
					bCanImproveAndEnableTogether = true
					break
			if bCanImproveAndEnableTogether:
				continue

			bRevealAndEnableTogether  = false
			for iRevealedBonus in liRevealedBonuses:
				if iBonus == iRevealedBonus:
					bRevealAndEnableTogether = true
					break
			if bRevealAndEnableTogether:
				continue

			screen.attachImageButton( panelName, "", gc.getBonusInfo(iBonus).getButton(), GenericButtonSizes.BUTTON_SIZE_CUSTOM, WidgetTypes.WIDGET_PEDIA_JUMP_TO_BONUS, iBonus, 1, False )
